#! /usr/bin/env python3.3
"""Functions for dealing with character encoding."""

from p4gf_l10n import NTR


def _encoding_list():
    """Return a list of character encodings, in preferred order.

    For use when attempting to read bytes of unknown encoding.

    """
    # latin_1 decodes anything, so keep it last in the list.
    return NTR(['utf8', 'latin_1'])


def decode(bites):
    """Attempt to decode using one of several code pages."""
    for encoding in _encoding_list():
        try:
            s = bites.decode(encoding)
            return s
        except UnicodeDecodeError:
            pass
    # Give up, re-create and raise the first error.
    bites.decode(_encoding_list[0])


def decode_p4_content(bites, vardict):
    """Decode the raw P4 file content using the available metadata.

    :param bytes bites: the raw file content from Perforce.
    :param dict vardict: the tagged file metadata (e.g. from p4 print).

    :return: decoded string.

    """
    if 'type' in vardict:
        if vardict['type'] == b'utf16':
            return bites.decode(encoding='utf-16')
    return bites.decode()
