#!/usr/bin/env bash

set -e

#check to make sure this is a 64-bit machine
getconf LONG_BIT | grep 64 > /dev/null  || ( echo "Git Fusion requires a 64-bit machine. Exiting." && exit 1)

echo ''
echo 'This script will upgrade P4Python to the latest 2014.1 build for Git Fusion'
echo 'It is not intended for use with package installs'
echo ''
echo "Do you wish to continue?"
select yn in "Yes" "No"; do
    case $yn in
        Yes ) break;;
        No ) exit;;
    esac
done

# Enforce /usr/local/bin for RHEL/Centos because the install script does. 
# All else can use what is found in the environment
PYTHON33="python3.3"
if [ -e "/etc/redhat-release" ]; then
	PYTHON33="/usr/local/bin/python3.3"
fi

if [ -e /opt/perforce/git-fusion/bin/python3.3 ]; then
	echo 'Found: /opt/perforce/git-fusion/bin/python3.3'
	echo 'Package install of Git Fusion detected. Use the package installer to upgrade P4Python'
	echo '   yum install perforce-git-fusion-p4python'
	exit 1
fi

PYTHON_VER=`$PYTHON33 -c 'import sys; print(sys.version_info[:])' | 
	awk -F'(' '{print $2}' | awk -F', ' '{print "Python "$1"."$2"."$3}'`
if [ "$PYTHON_VER" != "Python 3.3.2" ]; then
        echo 'Expected Python 3.3.2'
        echo 'Found: '$PYTHON_VER
	exit 1
fi

if [ ! -e build_p4python ]; then
        mkdir build_p4python
fi
cd build_p4python

echo import P4 > p4python_version_check.py
echo 'print(P4.P4.identify())' >> p4python_version_check.py
$PYTHON33 p4python_version_check.py
if [ $? -eq 1 ]; then
	echo 'There appears to be a problem with your existing P4Python installation.'
	exit 1
fi

echo 'Grabbing the required versions of P4Python and the P4API...'

wget http://ftp.perforce.com/perforce/r14.1/bin.tools/p4python.tgz
wget http://ftp.perforce.com/perforce/r14.1/bin.linux26x86_64/p4api.tgz

echo 'Building P4Python...'
tar xvzf p4python.tgz 
tar xvzf p4api.tgz
cd p4python-*
export PATH=.:$PATH
$PYTHON33 setup.py build --apidir ../p4api-2014.1.*/ --ssl

sudo $PYTHON33 setup.py install --apidir ../p4api-2014.1.*/ --ssl

echo 'Testing the installation...'
cd ..
$PYTHON33 p4python_version_check.py
cd ..
rm -rf build_p4python
